/**************************************************************************
 *
 * Copyright 2016 Samsung Electronics co., Ltd. All Rights Reserved.
 *
 * Contact: Konstantin Drabeniuk <k.drabeniuk@samsung.com>
 * Contact: Andrii Sokolenko <a.sokolenko@samsung.com>
 * Contact: Roman Marchenko <r.marchenko@samsung.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sub license, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
 * IN NO EVENT SHALL PRECISION INSIGHT AND/OR ITS SUPPLIERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
**************************************************************************/

#include "tc_tbm.h"

/* TODO::
 * 1. Test When there is no bufmgr.
 *
 */
TBMBufmgr::TBMBufmgr()
{
	bufmgr = NULL;
}

void TBMBufmgr::SetUp()
{
	TBMEnv::SetUp();

	bufmgr = tbm_bufmgr_init(-1);
	ASSERT_TRUE(bufmgr != NULL);
}

void TBMBufmgr::TearDown()
{
	tbm_bufmgr_deinit(bufmgr);

	ASSERT_TRUE(tbm_bufmgr_debug_get_ref_count() == 0);

	TBMEnv::TearDown();
}

static int memory_for_invalid_param;
static tbm_bufmgr invalid_bufmgr = (tbm_bufmgr)&memory_for_invalid_param;

/* tbm_bufmgr_get_capability() */
TEST_F(TBMBufmgr, BufmgrGetCapability)
{
	unsigned int capability;

	capability = tbm_bufmgr_get_capability(bufmgr);
	EXPECT_NE(capability, TBM_BUFMGR_CAPABILITY_NONE);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

TEST_F(TBMBufmgr, BufmgrGetCapabilityWithNullBufmgr)
{
	unsigned int capability;

	capability = tbm_bufmgr_get_capability(NULL);
	EXPECT_EQ(capability, TBM_BUFMGR_CAPABILITY_NONE);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMBufmgr, BufmgrGetCapabilityWithWrongBufmgr)
{
	tbm_bufmgr bufmgr = (tbm_bufmgr)TBM_UT_INVALID_VALUE;
	unsigned int capability;

	capability = tbm_bufmgr_get_capability(bufmgr);
	EXPECT_EQ(capability, TBM_BUFMGR_CAPABILITY_NONE);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

/* tbm_bufmgr_bind_native_display() */
#if 0 // TDDO:: fix the crash...
TEST_F(TBMBufmgr, BufmgrBindNativeDisplay)
{
	int ret;
	void *native_display = (void *)TBM_UT_INVALID_VALUE;

	ret = tbm_bufmgr_bind_native_display(bufmgr, native_display);
	ASSERT_EQ(ret, 1);
}

TEST_F(TBMBufmgr, BufmgrBindNativeDisplayWithNullBufmgr)
{
	int ret;
	void *native_display = (void *)TBM_UT_INVALID_VALUE;

	ret = tbm_bufmgr_bind_native_display(NULL, native_display);
	ASSERT_EQ(ret, 0);
}

TEST_F(TBMBufmgr, BufmgrBindNativeDisplayWithWrongBufmgr)
{
	int ret;
	void *native_display = (void *)TBM_UT_INVALID_VALUE;

	ret = tbm_bufmgr_bind_native_display(NULL, native_display);
	ASSERT_EQ(ret, 0);
}
#endif

/* tbm_bufmgr_set_bo_lock_type() */
TEST_F(TBMBufmgr, BufmgrSetBoLockType)
{
	int ret;

	ret = tbm_bufmgr_set_bo_lock_type(bufmgr, TBM_BUFMGR_BO_LOCK_TYPE_NEVER);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
	ret = tbm_bufmgr_set_bo_lock_type(bufmgr, TBM_BUFMGR_BO_LOCK_TYPE_ONCE);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
	ret = tbm_bufmgr_set_bo_lock_type(bufmgr, TBM_BUFMGR_BO_LOCK_TYPE_ALWAYS);
	EXPECT_EQ(ret, 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
}

TEST_F(TBMBufmgr, BufmgrSetBoLockTypeWithBufmgrParamTest)
{
	int ret;

	ret = tbm_bufmgr_set_bo_lock_type(NULL, TBM_BUFMGR_BO_LOCK_TYPE_NEVER);
	EXPECT_EQ(ret, 0);
	ret = tbm_bufmgr_set_bo_lock_type(invalid_bufmgr, TBM_BUFMGR_BO_LOCK_TYPE_NEVER);
	EXPECT_EQ(ret, 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

// TODO::::
/* tbm_bufmgr_debug_show() */
/* tbm_bufmgr_debug_tbm_info_get() */
/* tbm_bufmgr_debug_trace() */
/* tbm_bufmgr_debug_dump_all() */
/* tbm_bufmgr_debug_queue_dump() */
/* tbm_bufmgr_debug_dump_set_scale() */
/* tbm_bufmgr_debug_get_ref_count */

INSTANTIATE_TEST_CASE_P(TBMBufmgrParams,
						TBMBufmgr,
						Combine(Bool(), Bool(), Values("none")));

